package published

import (
    "errors"
    "database/sql/driver"
    "encoding/json"
)

type Cell struct {
    //id string `xml:"-"`
    Name            string `xml:"name,attr" json:"name"`
    Image           string `xml:"img,attr" json:"image"`
    Colspan        int `xml:"colspan,attr" json:"colSpan"`
    Rowspan        int `xml:"rowspan,attr" json:"rowSpan"`
    Action         string `xml:"action,attr" json:"action"`
    Title          string `xml:",innerxml" json:"title"`
    ShowTitle      int16 `xml:"showTitle,attr" json:"showTitle"`
    TextSize       int `xml:"textSize,attr" json:"textSize"`
    TextColor      string `xml:"textColor,attr" json:"textColor"`
    TextBackground string `xml:"textBackground,attr" json:"textBackground"`
    CellBackground string `xml:"cellBackground,attr" json:"cellBackground"`
}

type Row struct {
    //id string `xml:"-"`
    Height int `xml:"height,attr" json:"height"`
    Width  int `xml:"width,attr" json:"width"`
    Cells  []Cell `xml:"Cell" json:"cells"`
}

type Frame struct {
    //id string `xml:"-"`
    Title      string `xml:"title,attr" json:"title"`
    Icon       string `xml:"icon,attr" json:"icon"`
    Padding    int `xml:"padding,attr" json:"space"`
    CellWidth  int `xml:"cellWidth,attr" json:"cellWidth"`
    CellHeight int `xml:"cellHeight,attr" json:"cellHeight"`
    BgImage    string `xml:"bgImage,attr" json:"bgImage"`
    Rows       RowList `xml:"Row" json:"rows"`
}

type RowList []Row

func (p *RowList) Scan(src interface{}) error {
    source, ok := src.([]byte)
    if !ok {
        return errors.New("Type assertion .([]byte) failed.")
    }
    var i RowList
    err := json.Unmarshal(source, &i)
    if err != nil {
        return err
    }
    *p = i
    return nil
}

func (p RowList) Value() (driver.Value, error) {
    j, err := json.Marshal(p)
    return j, err
}

type Screen struct {
    Id          string `xml:"-"`
    Frames      []Frame `xml:"Frame" json:"frames"`
    Width       int `xml:"width,attr" json:"width"`
    Height      int `xml:"height,attr" json:"height"`
    Description string `xml:"description,attr" json:"description"`
}

type Image struct {
    //id string `xml:"-"`
    Source   string `xml:"src,attr" json:"source"`
    Mode     string `xml:"mode,attr" json:"mode"`
    Duration uint `xml:"duration,attr" json:"duration"`
    Text     string `xml:",innerxml" json:"text"`
}

type Media struct {
    //id string `xml:"-"`
    Images []Image `xml:"Image" json:"images"`
    Mode   string `xml:"mode,attr" json:"mode"`
    Loops  uint `xml:"loops,attr" json:"loops"`
}

func (p *Media) Scan(src interface{}) error {
    source, ok := src.([]byte)
    if !ok {
        return errors.New("Type assertion .([]byte) failed.")
    }
    var i Media
    err := json.Unmarshal(source, &i)
    if err != nil {
        return err
    }
    *p = i
    return nil
}

func (p Media) Value() (driver.Value, error) {
    j, err := json.Marshal(p)
    return j, err
}

type Splash struct {
    Id          string `xml:"-"`
    Media       Media `xml:"Media" json:"media"`
    //FitScreen string `xml:"fitScreen,attr" json:"fitScreen"`
    Description string `xml:"description" json:"description"`
}

type AppLayout struct {
    Etag   string  `xml:"etag,attr" json:"eTag"`
    Splash *Splash `xml:"Splash" json:"Splash"`
    Screen *Screen `xml:"Screen" json:"Screen"`
}
