package published

import (
    "errors"
    "encoding/json"
    "database/sql/driver"
    "time"
    "cygnux/kepler/models"
)

//type HStore map[string]string
//type HStoreList []HStore

type VodCategoryFilterTag struct {
    Name   string `json:"n"`
    Value  string `json:"v"`
    Remark string `json:"s"`
}

type VodCategoryFilter struct {
    Name       string `json:"ns"`
    Expression string `json:"exp"`
    Tags       []VodCategoryFilterTag `json:"tags"`
}

type VodCategoryFilterList []VodCategoryFilter

func (p *VodCategoryFilterList) Scan(src interface{}) error {
    source, ok := src.([]byte)
    if !ok {
        return errors.New("Type assertion .([]byte) failed.")
    }
    var i VodCategoryFilterList
    err := json.Unmarshal(source, &i)
    if err != nil {
        return err
    }
    *p = i
    return nil
}

func (p VodCategoryFilterList) Value() (driver.Value, error) {
    j, err := json.Marshal(p)
    return j, err
}

type VodCategory struct {
    Id           string `json:"id"`
    CategoryName string `json:"categoryName"`
    Index        uint `json:"index,omitempty"`
    Icon         string `json:"icon,omitempty"`
    Type         uint `json:"type,omitempty"`
    Filters      VodCategoryFilterList `json:"filters,omitempty"`
    Description  string `json:"description,omitempty"`
}

type VodAlbum struct {
    Id            string `json:"id"`
    AlbumName     string `json:"albumName"`
    IsSeries      bool `json:"isSeries"`
    FilmTotal     uint `json:"filmTotal"`
    FilmCount     uint `json:"filmCount"`
    FilmIndex     uint `json:"filmIndex"`
    Remark        string `json:"remark"`
    Duration      int `json:"duration"`
    Played        int64 `json:"played"`
    Viewed        int64 `json:"viewed"`
    AlbumPicture  string `json:"albumPicture"`
    Language      string `json:"language"`
    IssueDate     string `json:"issueDate,omitempty"`
    PublishTime   *time.Time `json:"publishTime,omitempty"`
    Score         float32 `json:"score,omitempty"`
    ProviderName  string `json:"providerName,omitempty"`
    Categories    models.HSTOREDictionary `json:"categories,omitempty"`
    Tags          models.StringArray `json:"tags,omitempty"`
    Geniuses      models.StringArray `json:"geniuses,omitempty"`
    Crews         models.JSONDictionary `json:"crews,omitempty"`
    Videos        models.JSONDictionaryArray `json:"videos,omitempty"`
    ScreenShots   models.StringArray `json:"screenShots,omitempty"`
    VIPInfo       models.JSONDictionary `json:"vipInfo,omitempty"`
    PlayControl   models.JSONDictionary `json:"playCtrl,omitempty"`
    Description   string `json:"description,omitempty"`
    Created       *time.Time `json:"created,omitempty"`
    Updated       *time.Time `json:"updated,omitempty"`
    Films         interface{} `json:"films,omitempty"`
    RelatedAlbums interface{} `json:"relatedAlbums,omitempty"`
}

type VodFilm struct {
    AlbumId     string `json:"albumId"`
    FilmId      string `json:"filmId"`
    FilmName    string `json:"filmName"`
    Index       int `json:"index"`
    Duration    int `json:"duration"`
    Played      int64 `json:"played"`
    Viewed      int64 `json:"viewed"`
    FilmPicture string `json:"filmPicture"`
    Remark      string `json:"remark"`
    Language    string `json:"language"`
    IssueDate   string `json:"issueDate,omitempty"`
    PlayControl models.JSONDictionary `json:"playCtrl,omitempty"`
    Crews       models.JSONDictionary `json:"crews,omitempty"`
    Videos      models.JSONDictionaryArray `json:"videos,omitempty"`
    ScreenShots models.StringArray `json:"screenShots,omitempty"`
    Description string `json:"description,omitempty"`
    Created     *time.Time `json:"created,omitempty"`
    Updated     *time.Time `json:"updated,omitempty"`
}

type VodRecommendAlbum struct {
    AlbumId    string `json:"albumId"`
    AlbumName  string `json:"albumName"`
    Score      float32 `json:"score,omitempty"`
    Remark     string `json:"remark"`
    Poster     string `json:"poster"`
    Categories models.HSTOREDictionary `json:"categories"`
}

